@@ -4,6 +4,7 @@ use full_moon::ast::{
     Call, Expression, FunctionArgs, FunctionBody, FunctionCall, FunctionDeclaration, FunctionName,
     LocalFunction, MethodCall, Parameter, Value,
 };
+use full_moon::node::Node;
 use full_moon::tokenizer::{Symbol, Token, TokenKind, TokenReference, TokenType};
 use std::borrow::Cow;
 use std::boxed::Box;
@@ -26,7 +27,7 @@ impl CodeFormatter {
         let additional_indent_level = self.get_range_indent_increase(function_token_range); //code_formatter.get_token_indent_increase(function_token.token());
 
         let function_token = crate::fmt_symbol!(self, function_token, "function");
-        let mut function_body = self.format_function_body(function_body);
+        let mut function_body = self.format_function_body(function_body, false);
 
         // Need to insert any additional trivia, as it isn't being inserted elsewhere
         #[cfg(feature = "luau")]
@@ -97,10 +98,13 @@ impl CodeFormatter {
                 let (start_parens, end_parens) = parentheses.tokens();
                 // Find the range of the function arguments
                 let function_call_range = (
-                    start_parens.end_position().bytes(),
-                    end_parens.start_position().bytes(),
+                    Token::end_position(&start_parens).bytes(),
+                    Token::start_position(&end_parens).bytes(),
                 );
-                let mut is_multiline = (function_call_range.1 - function_call_range.0) > 80; // TODO: Properly determine this arbitrary number, and see if other factors should come into play
+                let mut is_multiline =
+                    // We subtract 20 as we don't have full information about what preceded these function arguments (e.g. the function name).
+                    // This is used as a general estimate. TODO: see if we can improve this calculation
+                    self.get_indent_width() + (function_call_range.1 - function_call_range.0) > self.config.column_width - 20;
                 let current_arguments = arguments.pairs();
 
                 // Format all the arguments, so that we can prepare them and check to see whether they need expanding
@@ -124,6 +128,8 @@ impl CodeFormatter {
                         for argument in arguments.pairs() {
                             // Only check the leading and trailing trivia of the expression
                             // If the expression has inline comments, it should be handled elsewhere
+                            // Allow this, as this is what rustfmt creates
+                            #[allow(clippy::blocks_in_if_conditions)]
                             if trivia_util::get_expression_leading_trivia(argument.value())
                                 .iter()
                                 .chain(
@@ -156,7 +162,10 @@ impl CodeFormatter {
 
                 if is_multiline && !force_mutliline {
                     // If we only have one argument then we will not make it multi line (expanding it would have little value)
-                    if formatted_arguments.len() == 1 {
+                    // Unless, the argument is a hangable expression
+                    if formatted_arguments.len() == 1
+                        && !trivia_util::can_hang_expression(formatted_arguments.first().unwrap())
+                    {
                         is_multiline = false;
                     } else {
                         // Find how far we are currently indented, we can use this to determine when to expand
@@ -186,7 +195,7 @@ impl CodeFormatter {
                                             // We have a collapsed table constructor - add the width, and if it fails,
                                             // we need to expand
                                             width_passed += argument.to_string().len();
-                                            if width_passed > 80 {
+                                            if width_passed > self.config.column_width - 20 {
                                                 break;
                                             }
                                         }
@@ -199,7 +208,7 @@ impl CodeFormatter {
                                             break;
                                         }
                                         width_passed += argument.to_string().len();
-                                        if width_passed > 80 {
+                                        if width_passed > self.config.column_width - 20 {
                                             // We have passed 80 characters without a table or anonymous function
                                             // There is nothing else stopping us from expanding - so we will
                                             break;
@@ -221,8 +230,8 @@ impl CodeFormatter {
 
                     // Calculate to see if the end parentheses requires any additional indentation
                     let end_parens_additional_indent_level = self.get_range_indent_increase((
-                        end_parens.start_position().bytes(),
-                        end_parens.end_position().bytes(),
+                        Token::start_position(&end_parens).bytes(),
+                        Token::end_position(&end_parens).bytes(),
                     ));
                     let end_parens_leading_trivia =
                         vec![self.create_indent_trivia(end_parens_additional_indent_level)];
@@ -248,20 +257,60 @@ impl CodeFormatter {
                         self.format_symbol(end_parens, &end_parens_token),
                     );
 
-                    let mut formatted_arguments = Punctuated::new();
+                    let mut arguments = Punctuated::new();
+                    // Iterate through the original formatted_arguments, so we can see how the formatted version would look like
+                    let mut formatted_versions = formatted_arguments.iter();
 
                     self.add_indent_range(function_call_range);
 
                     for argument in current_arguments {
+                        // See what the formatted version of the argument would look like
+                        let formatted_version = formatted_versions
+                            .next()
+                            .expect("less arguments than expected");
+
                         let argument_range =
                             CodeFormatter::get_range_in_expression(argument.value());
                         let additional_indent_level =
                             self.get_range_indent_increase(argument_range);
 
+                        let indent_spacing = (self.indent_level
+                            + additional_indent_level.unwrap_or(0))
+                            * self.config.indent_width;
+                        let require_multiline_expression =
+                            trivia_util::can_hang_expression(argument.value())
+                                && indent_spacing
+                                    + formatted_version
+                                        .to_string()
+                                        .lines()
+                                        .next()
+                                        .expect("no lines")
+                                        .len()
+                                    > self.config.column_width;
+
+                        if require_multiline_expression {
+                            let expr_range = argument
+                                .range()
+                                .expect("no range for function call argument");
+                            self.add_indent_range((expr_range.0.bytes(), expr_range.1.bytes() + 1));
+                        }
+
                         // Unfortunately, we need to format again, taking into account in indent increase
                         // TODO: Can we fix this? We don't want to have to format twice
-                        let formatted_argument = trivia_formatter::expression_add_leading_trivia(
-                            self.format_expression(argument.value()),
+                        let mut formatted_argument = self.format_expression(argument.value());
+
+                        // Hang the expression if necessary
+                        if require_multiline_expression {
+                            formatted_argument = self.hang_expression_no_trailing_newline(
+                                formatted_argument,
+                                additional_indent_level,
+                                None,
+                            );
+                        }
+
+                        // Add the leading indent for the argument
+                        formatted_argument = trivia_formatter::expression_add_leading_trivia(
+                            formatted_argument,
                             FormatTriviaType::Append(vec![
                                 self.create_indent_trivia(additional_indent_level)
                             ]),
@@ -290,12 +339,12 @@ impl CodeFormatter {
                             ))),
                         };
 
-                        formatted_arguments.push(Pair::new(formatted_argument, punctuation))
+                        arguments.push(Pair::new(formatted_argument, punctuation))
                     }
 
                     FunctionArgs::Parentheses {
                         parentheses,
-                        arguments: formatted_arguments,
+                        arguments,
                     }
                 } else {
                     let parentheses = self.format_contained_span(&parentheses);
@@ -411,15 +460,62 @@ impl CodeFormatter {
     pub fn format_function_body<'ast>(
         &mut self,
         function_body: &FunctionBody<'ast>,
+        add_trivia: bool,
     ) -> FunctionBody<'ast> {
-        let parameters_parentheses =
-            self.format_contained_span(function_body.parameters_parentheses());
-        let formatted_parameters = self.format_parameters(function_body);
+        // Calculate trivia
+        let additional_indent_level = self
+            .get_range_indent_increase(CodeFormatter::get_token_range(function_body.end_token()));
+        let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
+        let trailing_trivia = vec![self.create_newline_trivia()];
+
+        let (formatted_parameters, multiline_params) = self.format_parameters(function_body);
+
+        let mut parameters_parentheses = match multiline_params {
+            true => {
+                // TODO: This is similar to multiline in FunctionArgs, can we resolve?
+                // Format start and end brace properly with correct trivia
+                let (start_parens, end_parens) = function_body.parameters_parentheses().tokens();
+
+                // Calculate to see if the end parentheses requires any additional indentation
+                let end_parens_additional_indent_level = self.get_range_indent_increase((
+                    Token::start_position(&end_parens).bytes(),
+                    Token::end_position(&end_parens).bytes(),
+                ));
+                let end_parens_leading_trivia = vec![
+                    self.create_newline_trivia(),
+                    self.create_indent_trivia(end_parens_additional_indent_level),
+                ];
+
+                // Add new_line trivia to start_parens
+                let start_parens_token = crate::fmt_symbol!(self, start_parens, "(");
+                let start_parens_token = trivia_formatter::token_reference_add_trivia(
+                    start_parens_token.into_owned(),
+                    FormatTriviaType::NoChange,
+                    FormatTriviaType::Append(vec![self.create_newline_trivia()]),
+                );
+
+                let end_parens_token = TokenReference::new(
+                    end_parens_leading_trivia,
+                    Token::new(TokenType::Symbol {
+                        symbol: Symbol::RightParen,
+                    }),
+                    vec![],
+                );
+
+                ContainedSpan::new(
+                    Cow::Owned(start_parens_token),
+                    self.format_symbol(end_parens, &end_parens_token),
+                )
+            }
+            false => self.format_contained_span(function_body.parameters_parentheses()),
+        };
 
         #[cfg(feature = "luau")]
         let mut type_specifiers;
         #[cfg(feature = "luau")]
         let return_type;
+        #[allow(unused_mut)]
+        let mut added_trailing_trivia = false;
 
         #[cfg(feature = "luau")]
         {
@@ -433,12 +529,40 @@ impl CodeFormatter {
             }
 
             return_type = match function_body.return_type() {
-                Some(return_type) => Some(self.format_type_specifier(return_type)),
+                Some(return_type) => Some({
+                    let formatted = self.format_type_specifier(return_type);
+                    if add_trivia {
+                        added_trailing_trivia = true;
+                        trivia_formatter::type_specifier_add_trailing_trivia(
+                            formatted,
+                            FormatTriviaType::Append(trailing_trivia.to_owned()),
+                        )
+                    } else {
+                        formatted
+                    }
+                }),
                 None => None,
             };
         }
 
-        let end_token = self.format_end_token(function_body.end_token());
+        if !added_trailing_trivia && add_trivia {
+            parameters_parentheses = trivia_formatter::contained_span_add_trivia(
+                parameters_parentheses,
+                FormatTriviaType::NoChange,
+                FormatTriviaType::Append(trailing_trivia.to_owned()),
+            )
+        }
+
+        let end_token = if add_trivia {
+            Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                self.format_end_token(function_body.end_token())
+                    .into_owned(),
+                FormatTriviaType::Append(leading_trivia),
+                FormatTriviaType::Append(trailing_trivia),
+            ))
+        } else {
+            self.format_end_token(function_body.end_token())
+        };
 
         let function_body = function_body
             .to_owned()
@@ -512,10 +636,20 @@ impl CodeFormatter {
         &mut self,
         function_declaration: &FunctionDeclaration<'ast>,
     ) -> FunctionDeclaration<'ast> {
-        let function_token =
-            crate::fmt_symbol!(self, function_declaration.function_token(), "function ");
+        // Calculate trivia
+        let additional_indent_level = self.get_range_indent_increase(
+            CodeFormatter::get_token_range(function_declaration.function_token()),
+        );
+        let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
+
+        let function_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
+            crate::fmt_symbol!(self, function_declaration.function_token(), "function ")
+                .into_owned(),
+            FormatTriviaType::Append(leading_trivia),
+            FormatTriviaType::NoChange,
+        ));
         let formatted_function_name = self.format_function_name(function_declaration.name());
-        let formatted_function_body = self.format_function_body(function_declaration.body());
+        let formatted_function_body = self.format_function_body(function_declaration.body(), true);
 
         FunctionDeclaration::new(formatted_function_name)
             .with_function_token(function_token)
@@ -527,10 +661,21 @@ impl CodeFormatter {
         &mut self,
         local_function: &LocalFunction<'ast>,
     ) -> LocalFunction<'ast> {
-        let local_token = crate::fmt_symbol!(self, local_function.local_token(), "local ");
+        // Calculate trivia
+        let additional_indent_level = self.get_range_indent_increase(
+            CodeFormatter::get_token_range(local_function.local_token()),
+        );
+        let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
+
+        let local_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
+            crate::fmt_symbol!(self, local_function.local_token(), "local ").into_owned(),
+            FormatTriviaType::Append(leading_trivia),
+            FormatTriviaType::NoChange,
+        ));
+
         let function_token = crate::fmt_symbol!(self, local_function.function_token(), "function ");
         let formatted_name = Cow::Owned(self.format_plain_token_reference(local_function.name()));
-        let formatted_function_body = self.format_function_body(local_function.func_body());
+        let formatted_function_body = self.format_function_body(local_function.func_body(), true);
 
         LocalFunction::new(formatted_name)
             .with_local_token(local_token)
@@ -558,23 +703,59 @@ impl CodeFormatter {
         }
     }
 
+    // Checks whether the input Parameter contains comments
+    fn parameter_contains_comments(parameter: &Parameter<'_>) -> bool {
+        match parameter {
+            Parameter::Ellipse(token) | Parameter::Name(token) => match token {
+                Cow::Owned(t) => trivia_util::token_contains_comments(&t),
+                Cow::Borrowed(t) => trivia_util::token_contains_comments(t),
+            },
+        }
+    }
     /// Utilises the FunctionBody iterator to format a list of Parameter nodes
+    /// Returns the formatted Punctuated sequence of parameters, and a bool indicating whether the parameters were forced onto multiple lines
     fn format_parameters<'ast>(
         &mut self,
         function_body: &FunctionBody<'ast>,
-    ) -> Punctuated<'ast, Parameter<'ast>> {
+    ) -> (Punctuated<'ast, Parameter<'ast>>, bool) {
         let mut formatted_parameters = Punctuated::new();
-        let mut parameters_iterator = function_body.parameters().iter().peekable();
-        while let Some(parameter) = parameters_iterator.next() {
-            let formatted_parameter = self.format_parameter(parameter);
-            let mut punctuation = None;
+        let force_multiline = function_body.parameters().pairs().any(|pair| {
+            pair.punctuation()
+                .map_or(false, |punc| trivia_util::token_contains_comments(punc))
+                || CodeFormatter::parameter_contains_comments(pair.value())
+        });
 
-            if parameters_iterator.peek().is_some() {
-                punctuation = Some(Cow::Owned(TokenReference::symbol(", ").unwrap()));
-            }
+        let parameters_iterator = function_body.parameters().pairs();
+        for pair in parameters_iterator {
+            let formatted_parameter = {
+                let param = self.format_parameter(pair.value());
+                if force_multiline {
+                    trivia_formatter::parameter_add_trivia(
+                        param,
+                        FormatTriviaType::Append(vec![self.create_indent_trivia(Some(1))]),
+                        FormatTriviaType::NoChange,
+                    )
+                } else {
+                    param
+                }
+            };
 
-            formatted_parameters.push(Pair::new(formatted_parameter, punctuation));
+            let formatted_punctuation = match pair.punctuation() {
+                Some(punctuation) => Some(match force_multiline {
+                    true => Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                        // Create a comma with no trailing space, and instead we will add a newline character
+                        crate::fmt_symbol!(self, punctuation, ",").into_owned(),
+                        FormatTriviaType::NoChange,
+                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
+                    )),
+                    // Create a comma, with a trailing space at the end
+                    false => crate::fmt_symbol!(self, punctuation, ", "),
+                }),
+                None => None,
+            };
+
+            formatted_parameters.push(Pair::new(formatted_parameter, formatted_punctuation));
         }
-        formatted_parameters
+        (formatted_parameters, force_multiline)
     }
 }
