@@ -4,6 +4,7 @@ use full_moon::ast::{
     span::ContainedSpan,
     Block,
 };
+use full_moon::node::Node;
 use full_moon::tokenizer::{
     StringLiteralQuoteType, Symbol, Token, TokenKind, TokenReference, TokenType,
 };
@@ -30,6 +31,8 @@ pub type Range = (usize, usize);
 pub struct CodeFormatter {
     /// The configuration passed to the formatter
     config: Config,
+    /// An optional range of values to format within the file.
+    range: Option<crate::Range>,
     /// The current indent level
     indent_level: usize,
     /// A link of specific ranges to indent increases. The indent increases are added ontop of indent_level
@@ -58,16 +61,75 @@ macro_rules! fmt_symbol {
     };
 }
 
+#[macro_export]
+macro_rules! check_should_format {
+    ($fmter:expr, $token:expr) => {
+        if !$fmter.should_format_node($token) {
+            return $token.to_owned();
+        }
+    };
+}
+
 impl CodeFormatter {
     /// Creates a new CodeFormatter, with the given configuration
-    pub fn new(config: Config) -> Self {
+    pub fn new(config: Config, range: Option<crate::Range>) -> Self {
         CodeFormatter {
             indent_level: 0,
             config,
+            range,
             indent_ranges: HashSet::new(),
         }
     }
 
+    /// Checks whether we should format the given node.
+    /// Firstly determines whether the node has an ignore comment present.
+    /// If not, checks whether the provided node is within the formatting range.
+    /// If not, the node should not be formatted.
+    pub fn should_format_node<'ast>(&self, node: &impl Node<'ast>) -> bool {
+        // Check comments
+        let leading_trivia = node.surrounding_trivia().0;
+        for trivia in leading_trivia {
+            let comment_lines = match trivia.token_type() {
+                TokenType::SingleLineComment { comment } => comment,
+                TokenType::MultiLineComment { comment, .. } => comment,
+                _ => continue,
+            }
+            .lines()
+            .map(|line| line.trim());
+
+            for line in comment_lines {
+                if line == "stylua: ignore" {
+                    return false;
+                }
+            }
+        }
+
+        if let Some(range) = self.range {
+            let mut in_range = true;
+
+            if let Some(start_bound) = range.start {
+                if let Some(node_start) = node.start_position() {
+                    if node_start.bytes() < start_bound {
+                        in_range = false;
+                    }
+                }
+            }
+
+            if let Some(end_bound) = range.end {
+                if let Some(node_end) = node.end_position() {
+                    if node_end.bytes() > end_bound {
+                        in_range = false;
+                    }
+                }
+            }
+
+            in_range
+        } else {
+            // No range provided, therefore always in formatting range
+            true
+        }
+    }
+
     /// Increase the level of indention at the current position of the formatter
     pub fn increment_indent_level(&mut self) {
         self.indent_level += 1;
@@ -540,7 +602,7 @@ impl CodeFormatter {
 }
 
 /// Continues mutating a Vec of Tokens until there is no more trailing whitespace present
-fn pop_until_no_whitespace<'ast>(trivia: &mut Vec<Token<'ast>>) {
+fn pop_until_no_whitespace(trivia: &mut Vec<Token>) {
     if let Some(t) = trivia.pop() {
         match t.token_kind() {
             TokenKind::Whitespace => pop_until_no_whitespace(trivia), // Keep popping until no more whitespace
@@ -562,6 +624,8 @@ impl<'ast> VisitorMut<'ast> for CodeFormatter {
 
     // Remove any extra whitespace at the end of the file
     fn visit_eof(&mut self, node: TokenReference<'ast>) -> TokenReference<'ast> {
+        check_should_format!(self, &node);
+
         // Need to preserve any comments in leading_trivia if present
         // The indent level will be 0 at this point, as we have finished the whole file, so we need to one-index it again
         self.indent_level += 1;
